class Notify(object):
    def __init__(self, batch):
        self.batch = batch
        self.channels = {}

    def subscribe(self, channel, callback):
        if not channel in self.channels:
            self.batch.wrapper.log.debug("creating new notification channel '%s'" % channel)
            self.channels[channel] = []

        if not callback in self.channels[channel]:
            self.channels[channel].append(callback)

    def notify(self, channel, arg):
        if not self.channels.get(channel):
            self.batch.wrapper.log.warn("trying to pass message '%s' to nonexistent channel %s" % (arg, channel))
        else:
            for callback in self.channels[channel]:
                callback(arg)



from dexy.reporter import Reporter
from jinja2 import Environment
from jinja2 import FileSystemLoader
import datetime
import operator
import os
import shutil

class RunReporter(Reporter):
    """
    Returns info about a dexy run.
    """
    ALLREPORTS = True
    ALIASES = ['run']

    def run(self, wrapper):
        latest_report_dir = os.path.join(wrapper.log_dir, 'run-latest')

        timestamp = datetime.datetime.now().strftime("%Y-%m-%d--%H-%M-%S")
        report_dir = os.path.join(wrapper.log_dir, "run-%s" % timestamp)
        report_filename = os.path.join(report_dir, 'index.html')

        # Remove any existing directory (unlikely)
        shutil.rmtree(report_dir, ignore_errors=True)

        # Copy template files (e.g. .css files)
        template_dir = os.path.join(os.path.dirname(__file__), 'files')
        shutil.copytree(template_dir, report_dir)

        env_data = {}

        env_data['float'] = float
        env_data['len'] = len
        env_data['sorted'] = sorted
        env_data['hasattr'] = hasattr
        env_data['dict'] = dict
        env_data['isinstance'] = isinstance
        env_data['attrgetter'] = operator.attrgetter

        env_data['batch'] = wrapper.batch

        env_data['docs'] = wrapper.batch.docs()
        env_data['wrapper'] = wrapper

        env = Environment()
        env.loader = FileSystemLoader(os.path.dirname(__file__))
        template = env.get_template('template.html')

        template.stream(env_data).dump(report_filename, encoding="utf-8")

        # Copy this to run-latest
        # TODO symlink instead?
        shutil.rmtree(latest_report_dir, ignore_errors=True)
        shutil.copytree(report_dir, latest_report_dir)



from dexy.notify import Notify
from dexy.tests.utils import wrap

class Subscriber(object):
    def call_me(self, args):
        print "I received call_me with %s" % args

def test_notify():
    with wrap() as wrapper:
        wrapper.setup_batch()
        notifier = Notify(wrapper.batch)
        obj = Subscriber()
        notifier.subscribe("foo", obj.call_me)
        notifier.notify("foo", "bar")



from dexy.tests.utils import wrap
from dexy.doc import Doc

def test_split_html_filter():
    with wrap() as wrapper:
        contents="""
        <p>This is at the top.</p>
        <!-- split "a-page" -->
        some content on a page
        <!-- split "another-page" -->
        some content on another page
        <!-- endsplit -->
        bottom
        """

        doc = Doc("subdir/example.html|splithtml", contents=contents, wrapper=wrapper)
        wrapper.run_docs(doc)

        assert doc.children[2].key == "subdir/a-page.html"
        assert doc.children[3].key == "subdir/another-page.html"

        od = doc.output().data()

        assert "<p>This is at the top.</p>" in od
        assert '<a href="a-page.html">' in od
        assert '<a href="another-page.html">' in od
        assert "bottom" in od

        assert "<p>This is at the top.</p>" in doc.children[2].output().data()
        assert "some content on a page" in doc.children[2].output().data()
        assert "bottom" in doc.children[2].output().data()

        assert "<p>This is at the top.</p>" in doc.children[3].output().data()
        assert "some content on another page" in doc.children[3].output().data()
        assert "bottom" in doc.children[3].output().data()

def test_split_html_additional_filters():
    with wrap() as wrapper:
        contents="""
        <p>This is at the top.</p>
        <!-- split "a-page" -->
        some content on a page
        <!-- split "another-page" -->
        some content on another page
        <!-- endsplit -->
        bottom
        """

        doc = Doc("example.html|splithtml",
                contents=contents,
                splithtml = { "keep-originals" : False, "additional-doc-filters" : "processtext" },
                wrapper=wrapper
              )
        wrapper.run_docs(doc)

        assert doc.children[2].key == "a-page.html|processtext"
        assert doc.children[3].key == "another-page.html|processtext"

        od = doc.output().data()
        assert "<p>This is at the top.</p>" in od
        assert '<a href="a-page.html">' in od
        assert '<a href="another-page.html">' in od
        assert "bottom" in od

        assert "<p>This is at the top.</p>" in doc.children[2].output().data()
        assert "some content on a page" in doc.children[2].output().data()
        assert "bottom" in doc.children[2].output().data()
        assert "Dexy processed the text" in doc.children[2].output().data()

        assert "<p>This is at the top.</p>" in doc.children[3].output().data()
        assert "some content on another page" in doc.children[3].output().data()
        assert "bottom" in doc.children[3].output().data()
        assert "Dexy processed the text" in doc.children[3].output().data()



from dexy.common import OrderedDict
from dexy.data import Data
from dexy.doc import Doc
from dexy.doc import PatternDoc
from dexy.exceptions import UserFeedback
from dexy.filter import DexyFilter
from dexy.tests.utils import wrap
from nose.tools import raises

@raises(UserFeedback)
def test_blank_alias():
    with wrap() as wrapper:
        doc = Doc("abc.txt|", wrapper=wrapper)
        doc.populate()

@raises(UserFeedback)
def test_blank_alias_exception():
    doc = Doc("foo")
    doc.filter_class_for_alias("")

def test_filter_class_for_alias():
    doc = Doc("foo")
    filter_class = doc.filter_class_for_alias("dexy")
    assert filter_class == DexyFilter

def test_output_is_data():
    with wrap() as wrapper:
        doc = Doc("abc.txt", contents="these are the contents", wrapper=wrapper)
        wrapper.run_docs(doc)
        assert isinstance(doc.output(), Data)

def test_create_doc_with_child():
    with wrap() as wrapper:
        doc = Doc("parent.txt", Doc("child.txt", wrapper=wrapper), wrapper=wrapper)
        doc.populate()
        assert doc.key == "parent.txt"
        assert doc.children[0].key == "child.txt"
        assert doc.children[1].key == "parent.txt"

def test_create_virtual_initial_artifact():
    with wrap() as wrapper:
        doc = Doc("abc.txt", contents="these are the contents", wrapper=wrapper)
        wrapper.run_docs(doc)
        assert doc.children[0].__class__.__name__ == "InitialVirtualArtifact"
        assert doc.children[0].output_data.__class__.__name__ == "Generic"

def test_create_virtual_initial_artifact_with_dict():
    with wrap() as wrapper:
        od_contents = OrderedDict()
        od_contents['1'] = "these are the contents"
        doc = Doc("abc.txt", contents = od_contents, wrapper=wrapper)
        wrapper.run_docs(doc)
        assert doc.children[0].output_data.__class__.__name__ == "Sectioned"

def test_create_doc_with_filters():
    with wrap() as wrapper:
        doc = Doc("abc.txt|outputabc", contents="these are the contents", wrapper=wrapper)
        wrapper.run_docs(doc)

def test_doc_setup():
    with wrap() as wrapper:
        with open("abc.txt", "w") as f:
            f.write("def")

        doc = Doc("abc.txt|dexy|dexy", wrapper=wrapper)

        doc.populate()

        for child in doc.children:
            child.setup()

        doc.setup()

        assert doc.key == "abc.txt|dexy|dexy"
        assert doc.name == "abc.txt"
        assert doc.filters == ["dexy", "dexy"]

        assert doc.children[0].key == "abc.txt"
        assert doc.children[1].key == "abc.txt|dexy"
        assert doc.children[2].key == "abc.txt|dexy|dexy"

        assert doc.children[0].__class__.__name__ == "InitialArtifact"
        assert doc.children[1].__class__.__name__ == "FilterArtifact"
        assert doc.children[2].__class__.__name__ == "FilterArtifact"

        assert not hasattr(doc.children[0], 'next_filter_alias')
        assert doc.children[1].next_filter_alias == "dexy"
        assert doc.children[2].next_filter_alias == None

        assert not doc.children[0].prior
        assert doc.children[1].prior.__class__.__name__ == "InitialArtifact"
        assert doc.children[2].prior.__class__.__name__ == "FilterArtifact"

        assert not doc.children[0].prior
        assert doc.children[1].prior.key == "abc.txt"
        assert doc.children[2].prior.key == "abc.txt|dexy"

def test_setup_pattern_doc_no_filters():
    with wrap() as wrapper:
        doc = PatternDoc("*.txt", wrapper=wrapper)
        doc.populate()
        doc.setup()
        assert doc.file_pattern == "*.txt"
        assert doc.filter_aliases == []

def test_setup_pattern_doc_one_filter():
    with wrap() as wrapper:
        doc = PatternDoc("*.txt|dexy", wrapper=wrapper)
        doc.populate()
        doc.setup()
        assert doc.file_pattern == "*.txt"
        assert doc.filter_aliases == ['dexy']

def test_setup_pattern_doc_many_filters():
    with wrap() as wrapper:
        doc = PatternDoc("*.txt|dexy|dexy|dexy", wrapper=wrapper)
        doc.populate()
        doc.setup()
        assert doc.file_pattern == "*.txt"
        assert doc.filter_aliases == ['dexy', 'dexy', 'dexy']

def test_pattern_doc_args():
    with wrap() as wrapper:
        with open("hello.txt", "w") as f:
            f.write("hello!")

        wrapper.setup_batch()

        doc = PatternDoc("*.txt", foo="bar", wrapper=wrapper)
        doc.populate()
        assert doc.args['foo'] == 'bar'
        assert isinstance(doc.children[0], Doc)
        assert doc.children[0].key == "hello.txt"
        assert doc.children[0].args['foo'] == 'bar'
